#import "qaccessibleuielement_mac_p.h"

#import <AppKit/NSAccessibility.h>

#include <qaccessible.h>
#include <qdebug.h>

NSString *macRoleForInterface(QAccessibleInterface *interface);

@implementation QAccessibleUIElement

#define MAC_ACCESSIBILTY_DEVELOPER_MODE

- (id)initWithAccessibleInterface:(QAccessibleInterface *)aInterface parent:(id)aParent {
    if ((self = [super init])) {
        role = macRoleForInterface(aInterface);
        interface = aInterface;
        parent = [aParent retain];
    }
    return self;
}

+ (QAccessibleUIElement *)elementWithAccessibleInterface:(QAccessibleInterface *)aInterface parent:(id)aParent {
    return [[[self alloc] initWithAccessibleInterface:aInterface parent:aParent] autorelease];
}

- (void)dealloc {
    delete interface;
    [parent autorelease];
    [super dealloc];
}

- (BOOL)isEqual:(id)object {
    if ([object isKindOfClass:[QAccessibleUIElement self]]) {
        QAccessibleUIElement *other = object;
        return [role isEqualToString:other->role] && [parent isEqual:other->parent];
    } else {
        return NO;
    }
}

- (NSUInteger)hash {
    // Equal objects must hash the same.
    return qHash(interface->object());
}

//
// accessibility protocol
//

// attributes

- (NSArray *)accessibilityAttributeNames {
    static NSArray *attributes = nil;
    if (attributes == nil) {
	attributes = [[NSArray alloc] initWithObjects:
	    NSAccessibilityRoleAttribute,
	    NSAccessibilityRoleDescriptionAttribute,
            NSAccessibilityChildrenAttribute,
#ifdef MAC_ACCESSIBILTY_DEVELOPER_MODE
            NSAccessibilityDescriptionAttribute,
#endif
	    NSAccessibilityFocusedAttribute,
	    NSAccessibilityParentAttribute,
	    NSAccessibilityWindowAttribute,
	    NSAccessibilityTopLevelUIElementAttribute,
	    NSAccessibilityPositionAttribute,
	    NSAccessibilitySizeAttribute,
	    nil];
    }
    return attributes;
}

- (id)accessibilityAttributeValue:(NSString *)attribute {
    if ([attribute isEqualToString:NSAccessibilityRoleAttribute]) {
	return role;
    } else if ([attribute isEqualToString:NSAccessibilityRoleDescriptionAttribute]) {
	return NSAccessibilityRoleDescription(role, nil);
    } else if ([attribute isEqualToString:NSAccessibilityChildrenAttribute]) {
//        NSMutableArray *kids = [NSMutableArray arrayWithCapacity: 0];
//        return kids;

        int childCount = interface->childCount();
        //qDebug() << "widget" << interface->object() << childCount;
        NSMutableArray *kids = [NSMutableArray arrayWithCapacity: 0];
        for (int i = 0; i < childCount; ++i) {
          //  qDebug() << "testing" <<  i;
            QAccessibleInterface *child = 0;
            int ret = interface->navigate(QAccessible::Child, i + 1, &child);
            if (child && ret == 0) {
                QAccessibleUIElement *uielement = [QAccessibleUIElement elementWithAccessibleInterface:child parent:self];
//                qDebug() << "add kid"<< i << uielement;
                [kids addObject: uielement];
            } else {
                delete child;
            }
        }
        return kids;
        //return NSAccessibilityUnignoredChildren(kids);

#ifdef MAC_ACCESSIBILTY_DEVELOPER_MODE
    } else if ([attribute isEqualToString: NSAccessibilityDescriptionAttribute]) {
        QObject *object = interface->object();
        if (object) {
            const char *className = object->metaObject()->className();
            return [NSString stringWithUTF8String : className];
        } else {
            return @"(no object)";
        }
#endif
    } else if ([attribute isEqualToString:NSAccessibilityFocusedAttribute]) {
        // Just check if the app thinks we're focused.
        id focusedElement = [NSApp accessibilityAttributeValue:NSAccessibilityFocusedUIElementAttribute];
        return [NSNumber numberWithBool:[focusedElement isEqual:self]];
    } else if ([attribute isEqualToString:NSAccessibilityParentAttribute]) {
        return NSAccessibilityUnignoredAncestor(parent);
    } else if ([attribute isEqualToString:NSAccessibilityWindowAttribute]) {
        // We're in the same window as our parent.
        return [parent accessibilityAttributeValue:NSAccessibilityWindowAttribute];
    } else if ([attribute isEqualToString:NSAccessibilityTopLevelUIElementAttribute]) {
        // We're in the same top level element as our parent.
        return [parent accessibilityAttributeValue:NSAccessibilityTopLevelUIElementAttribute];
    } else if ([attribute isEqualToString:NSAccessibilityPositionAttribute]) {
        QRect rect = interface->rect(0);
        // qDebug() << "rect" << rect << rect.x()<< rect.y();
        return [NSValue valueWithPoint: NSMakePoint(rect.x(), rect.y())];
    } else if ([attribute isEqualToString:NSAccessibilitySizeAttribute]) {
        QRect rect = interface->rect(0);
        return [NSValue valueWithSize: NSMakeSize(rect.width(), rect.height())];
    } else {
        return nil;
    }
}

- (BOOL)accessibilityIsAttributeSettable:(NSString *)attribute {
    if ([attribute isEqualToString:NSAccessibilityFocusedAttribute]) {
        // return [parent isFauxUIElementFocusable:self];
        return YES;
    } else {
        return NO;
    }
}

- (void)accessibilitySetValue:(id)value forAttribute:(NSString *)attribute {
    if ([attribute isEqualToString:NSAccessibilityFocusedAttribute]) {
        // [parent fauxUIElement:self setFocus:value];
    }
}


// actions

- (NSArray *)accessibilityActionNames {
    return [NSArray array];
}

- (NSString *)accessibilityActionDescription:(NSString *)action {
    Q_UNUSED(action);
    return nil;
}

- (void)accessibilityPerformAction:(NSString *)action {
    Q_UNUSED(action);
}


// misc

- (BOOL)accessibilityIsIgnored {
    return NO;
}

- (id)accessibilityHitTest:(NSPoint)point {
    Q_UNUSED(point);
    return NSAccessibilityUnignoredAncestor(self);
}

- (id)accessibilityFocusedUIElement {
    return NSAccessibilityUnignoredAncestor(self);
}

@end
