/****************************************************************************
**
** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
** All rights reserved.
** Contact: Nokia Corporation (qt-info@nokia.com)
**
** This file is part of the plugins of the Qt Toolkit.
**
** $QT_BEGIN_LICENSE:LGPL$
** No Commercial Usage
** This file contains pre-release code and may not be distributed.
** You may use this file in accordance with the terms and conditions
** contained in the Technology Preview License Agreement accompanying
** this package.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Nokia gives you certain additional
** rights.  These rights are described in the Nokia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** If you have questions regarding the use of this file, please contact
** Nokia at qt-info@nokia.com.
**
**
**
**
**
**
**
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include "qaccessibledeclarativeobject.h"
#include <qnamespace.h>

#ifndef QT_NO_ACCESSIBILITY

QT_BEGIN_NAMESPACE


/*
    Creates a new QAccessibleDeclarativeObject instance bases on the objects
    accessibleRole. Takes ownership of the implementation object.
*/
QAccessibleDeclarativeObject *QAccessibleDeclarativeObject::instantiateObject(QObject *object, QAccessibleDeclarativeImplementation *implementation)
{
    const QVariant role = QDeclarativeAccessibleAttached::property(object, "role");
    const QAccessible::Role roleAttribute = static_cast<QAccessible::Role>(role.toInt());

    QAccessibleDeclarativeObject *accessibleObject = 0;

    // Add code here to map roleAttributes to QAccessibleRoles and
    // QAccessibleDeclarativeObject subclasses.
    if (0) {
    } else if (roleAttribute == QAccessible::PushButton) {
        accessibleObject = new QAccessibleDeclarativeObject(object, implementation, QAccessible::PushButton);
    } else if (roleAttribute == QAccessible::StaticText) {
        accessibleObject = new QAccessibleDeclarativeObject(object, implementation, QAccessible::StaticText);
    } else if (roleAttribute == QAccessible::CheckBox) {
        accessibleObject = new QAccessibleDeclarativeObject(object, implementation, QAccessible::CheckBox);
    } else if (roleAttribute == QAccessible::RadioButton) {
        accessibleObject = new QAccessibleDeclarativeObject(object, implementation, QAccessible::RadioButton);
    } else if (roleAttribute == QAccessible::EditableText) {
        accessibleObject = new QAccessibleDeclarativeEditableTextObject(object, implementation, QAccessible::EditableText);

    /*
    TODO
    } else if (roleAttribute == QLatin1String("ProgressBar")) {
       accessibleObject = new QAccessibleDeclarativeObject(object, implementation, QAccessible::ProgressBar);
    } else if (roleAttribute == QLatin1String("Slider")) {
       accessibleObject = new QAccessibleDeclarativeObject(object, implementation, QAccessible::Slider);
*/
    } else {
        // This handles unknown roles and items with no role set.
        accessibleObject = new QAccessibleDeclarativeObject(object, implementation, QAccessible::Pane);
    }

    // the graphicsview/scene graph implementation might need to call down to
    // QAccessibleObjectEx, give it a pointer (we still own the object).
    implementation->setBaseObject(accessibleObject);

    return accessibleObject;
}

QAccessibleDeclarativeObject::QAccessibleDeclarativeObject(QObject *object, QAccessibleDeclarativeImplementation *implementation, QAccessible::Role role)
    :QAccessibleObjectEx(object)
    ,m_implementation(implementation)
    ,m_role(role)
{

}

QAccessibleDeclarativeObject::~QAccessibleDeclarativeObject()
{
    delete m_implementation;
}

bool QAccessibleDeclarativeObject::hasChildren() const
{
    if (m_role == PushButton ||
        m_role == CheckBox ||
        m_role == RadioButton ||
        m_role == EditableText) {
        return false;
    }
    return true;
}

QRect QAccessibleDeclarativeObject::rect(int child) const
{
    return m_implementation->itemRect();
}

int QAccessibleDeclarativeObject::childCount() const
{
    if (!hasChildren())
       return 0;

    if (!m_implementation->isAccessible())
        return 0;

    return m_implementation->childCount();
}

int QAccessibleDeclarativeObject::navigate(RelationFlag rel, int entry, QAccessibleInterface **target) const
{
    if (rel == QAccessible::Child && !hasChildren()) {
        *target = 0;
        return -1;
    }

    return m_implementation->navigate(rel, entry, target);
}

int QAccessibleDeclarativeObject::indexOfChild(const QAccessibleInterface *iface) const
{
    if (iface) {
        return m_implementation->indexOfChild(iface);
    }
    return -1;

}

QFlags<QAccessible::RelationFlag> QAccessibleDeclarativeObject::relationTo(int, const QAccessibleInterface*, int) const
{
    return 0;
}

int QAccessibleDeclarativeObject::childAt(int x, int y) const
{
    // Look for children first.
    // Start with the last child first, because children are ordered in paint order
    // (which is opposite of hit test order)
    int hitChild = -1;
    // If the item clips its children, we can return early if the coordinate is outside its rect
    if (m_implementation->clipsChildren()) {
        if (!m_implementation->itemRect().contains(x, y))
            return -1;
    }
    for (int i = childCount(); i > 0 && hitChild == -1; --i) {
        QAccessibleInterface *child = 0;
        int id = navigate(QAccessible::Child, i, &child);
        if (child && id == 0) {
            if (!(child->state(0) & QAccessible::Invisible)) {
                if (child->childAt(x, y) != -1)
                    hitChild = i;
            }
            delete child;
        }
    }

    if (hitChild == -1) {
        if (m_implementation->itemRect().contains(x, y))
            hitChild = 0;
    }

    return hitChild;
}

QString QAccessibleDeclarativeObject::text(QAccessible::Text textType, int) const
{
    // handles generic behaviour not specific to an item
    switch (textType) {
    case QAccessible::DebugDescription:
        return object()->metaObject()->className();
        break;
    case QAccessible::Name: {
        QVariant accessibleName = QDeclarativeAccessibleAttached::property(object(), "name");
        if (!accessibleName.isNull())
            return accessibleName.toString();
        break;}
    case QAccessible::Description: {
        QVariant accessibleDecription = QDeclarativeAccessibleAttached::property(object(), "description");
        if (!accessibleDecription.isNull())
            return accessibleDecription.toString();
        break;}
    case Value:
    case Help:
    case Accelerator:
    default:
        break;
    }


    // the following blocks handles item-specific behaviour
    if (m_role == QAccessible::EditableText) {
        if (textType == QAccessible::Value) {
            QVariant text = object()->property("text");
            return text.toString();
        } else if (textType == QAccessible::Name) {
            return object()->objectName();
        }
    } else {
        if (textType == QAccessible::Name) {
            QVariant text = object()->property("text");
            return text.toString();
        }
    }


    return QString();
}

QAccessible::Role QAccessibleDeclarativeObject::role(int) const
{
    return m_role;
}

QFlags<QAccessible::StateFlag> QAccessibleDeclarativeObject::state(int) const
{
    QAccessible::State state = m_implementation->state();

    //QRect viewRect(QPoint(0, 0), m_implementation->size());
    //QRect itemRect(m_item->scenePos().toPoint(), m_item->boundingRect().size().toSize());

    QRect viewRect = m_implementation->viewRect();
    QRect itemRect = m_implementation->itemRect();

   // qDebug() << "viewRect" << viewRect << "itemRect" << itemRect;
    // error case:
    if (viewRect.isNull() || itemRect.isNull()) {
        state |= QAccessible::Invisible;
    }

    if (!viewRect.intersects(itemRect)) {
        state |= QAccessible::Offscreen;
        // state |= QAccessible::Invisible; // no set at this point to ease development
    }

    if (!object()->property("visible").toBool() || qFuzzyIsNull(object()->property("opacity").toDouble())) {
        state |= QAccessible::Invisible;
    }

    if ((m_role == CheckBox || m_role == RadioButton) && object()->property("checked").toBool()) {
        state |= QAccessible::Checked;
    }

    if (m_role == QAccessible::EditableText)
        state |= QAccessible::Focusable;

    //qDebug() << "state?" << m_item->property("state").toString() << m_item->property("status").toString() << m_item->property("visible").toString();

    return state;
}

bool QAccessibleDeclarativeObject::doAction(int action, int child, const QVariantList &params)
{
    if (m_role == PushButton && action == QAccessible::Press) {
        QMetaObject::invokeMethod(object(), "accessibleAction", Q_ARG(QVariant, action));
    }
    if ((m_role == CheckBox || m_role == RadioButton) && action == QAccessible::Press) {
        bool checked = object()->property("checked").toBool();
        object()->setProperty("checked",  QVariant(!checked));
    }

    return QAccessibleObjectEx::doAction(action, child, params);
}

QVariant QAccessibleDeclarativeObject::invokeMethodEx(QAccessible::Method method, int, const QVariantList&)
{
    return QVariant();
}


// TEXT interface

QAccessibleDeclarativeTextObject::QAccessibleDeclarativeTextObject(QObject *object, QAccessibleDeclarativeImplementation *implementation, QAccessible::Role role)
    : QAccessibleDeclarativeObject(object, implementation, role)
{
}

QFlags<QAccessible::StateFlag> QAccessibleDeclarativeTextObject::state(int child) const
{
    Q_ASSERT(child == 0);
    return QAccessibleDeclarativeObject::state(child) | HasInvokeExtension;
}

QString QAccessibleDeclarativeTextObject::text() const
{
    QVariant text = object()->property("text");
    return text.toString();
}

void QAccessibleDeclarativeTextObject::setText(QAccessible::Text t, int child, const QString& text)
{
    Q_ASSERT(child == 0);
    // do we care about the type?
    object()->setProperty("text", text);
}

int QAccessibleDeclarativeTextObject::characterCount()
{
    return text().length();
}

QString QAccessibleDeclarativeTextObject::text(int startOffset, int endOffset)
{
    // FIXME: password mode in all of these?
    return text().mid(startOffset, endOffset - startOffset);
}



QString QAccessibleDeclarativeTextObject::textAfterOffset(int offset, QAccessible2::BoundaryType boundaryType, int* startOffset, int* endOffset)
{
    return textAfterOffsetFromString(offset, boundaryType, startOffset, endOffset, text());
}

QString QAccessibleDeclarativeTextObject::textAtOffset(int offset, QAccessible2::BoundaryType boundaryType, int* startOffset, int* endOffset)
{
    return textAtOffsetFromString(offset, boundaryType, startOffset, endOffset, text());
}

QString QAccessibleDeclarativeTextObject::textBeforeOffset(int offset, QAccessible2::BoundaryType boundaryType, int* startOffset, int* endOffset)
{
    return textBeforeOffsetFromString(offset, boundaryType, startOffset, endOffset, text());
}

int QAccessibleDeclarativeTextObject::offsetAtPoint(const QPoint& point, QAccessible2::CoordinateType coordType)
{
    // TODO
    return 0;
}
QString QAccessibleDeclarativeTextObject::attributes(int offset, int* startOffset, int* endOffset)
{
    // TODO
    return QString();
}
QRect QAccessibleDeclarativeTextObject::characterRect(int offset, QAccessible2::CoordinateType coordType)
{
    // TODO
    return QRect();
}


int QAccessibleDeclarativeTextObject::cursorPosition()
{
    return 0;
}
void QAccessibleDeclarativeTextObject::setCursorPosition(int position)
{

}
void QAccessibleDeclarativeTextObject::scrollToSubstring(int startIndex, int endIndex)
{

}

int QAccessibleDeclarativeTextObject::selectionCount()
{
    return 0;
}
void QAccessibleDeclarativeTextObject::selection(int selectionIndex, int* startOffset, int* endOffset)
{

}
void QAccessibleDeclarativeTextObject::setSelection(int selectionIndex, int startOffset, int endOffset)
{

}
void QAccessibleDeclarativeTextObject::addSelection(int startOffset, int endOffset)
{

}
void QAccessibleDeclarativeTextObject::removeSelection(int selectionIndex)
{

}

QT_END_NAMESPACE

#endif // QT_NO_ACCESSIBILITY
