#include "qaccessiblegraphicsviewimplementation.h"
#include "qaccessibledeclarativeobject.h"

QAccessibleGraphicsViewImplementation::QAccessibleGraphicsViewImplementation(QGraphicsItem *item, QGraphicsView *view)
    :QAccessibleDeclarativeImplementation()
    ,m_item(item)
    ,m_view(view)
{

}


QAccessibleDeclarativeObject *QAccessibleGraphicsViewImplementation::instantiateObject(QGraphicsObject *item, QGraphicsView *view)
{
    QAccessibleGraphicsViewImplementation * impl = new QAccessibleGraphicsViewImplementation(item, view);
    return QAccessibleDeclarativeObject::instantiateObject(item, impl);
}


int QAccessibleGraphicsViewImplementation::childCount() const
{
    QList<QGraphicsItem *> children = m_item->childItems();
    return children.count();
}

QRect QAccessibleGraphicsViewImplementation::itemRect() const
{
    QRectF sceneRect = m_item->sceneTransform().mapRect(m_item->boundingRect());
    QPoint pos = m_view->mapFromScene(m_view->mapToGlobal(sceneRect.topLeft().toPoint()));
    QSize size = sceneRect.size().toSize();
    return QRect(pos, size);
}

QRect QAccessibleGraphicsViewImplementation::viewRect() const
{
    QPoint screenPos = m_view->mapToGlobal(m_view->pos());
    return QRect(screenPos, m_view->size());
}

bool QAccessibleGraphicsViewImplementation::clipsChildren() const
{
    return static_cast<QDeclarativeItem *>(m_item)->clip();
}

static inline bool isAncestor(const QObject *ancestorCandidate, const QObject *child)
{
    while (child) {
        if (child == ancestorCandidate)
            return true;
        child = child->parent();
    }
    return false;
}

int QAccessibleGraphicsViewImplementation::navigate(QAccessible::RelationFlag rel, int entry, QAccessibleInterface **target) const
{
    //qDebug() << "QAccessibleDeclarativeItem navigate" << rel << entry;
    Q_ASSERT(entry >= 0);

    *target = 0;
    if (entry == 0) {
        *target = instantiateObject(m_item->toGraphicsObject(), m_view);
        return 0;
    }

    switch (rel) {
    case QAccessible::Child: {
        QList<QGraphicsItem *> children = m_item->childItems();
        const int childIndex = entry - 1;

        if (childIndex >= children.count())
            return -1;

        QGraphicsItem *child = children.at(childIndex);
        QGraphicsObject *childObject = qobject_cast<QGraphicsObject *>(child);
        if (!childObject)
            return -1;

        *target = instantiateObject(childObject, m_view);
        return 0;
        break;}
    case QAccessible::Ancestor: {
        Q_ASSERT(entry >= 1);
        QGraphicsItem *parent = m_item->parentItem();
        QGraphicsObject *parentObj = parent ? parent->toGraphicsObject() : 0;
        if (parent && !parentObj)
            qWarning("Can not make QGraphicsItems accessible");
        QAccessibleInterface *ancestor = (parentObj
                 ? instantiateObject(parentObj, m_view)
                 : QAccessible::queryAccessibleInterface(m_view));
        if (entry == 1) {
            *target = ancestor;
            return 0;
        } else if (entry > 1) {
            int ret = ancestor->navigate(QAccessible::Ancestor, entry - 1, target);
            delete ancestor;
            return ret;
        }
        break;}
    case QAccessible::Sibling: {
        QAccessibleInterface *iface = 0;
        if (navigate(QAccessible::Ancestor, 1, &iface) == 0) {
            if (iface) {
                int ret = iface->navigate(QAccessible::Child, entry, target);
                delete iface;
                return ret;
            }
        }
        return -1;
        break;}
    case QAccessible::FocusChild: {
        QGraphicsObject *focusObject = 0;
        if (m_item->hasFocus()) {
            focusObject = m_item->toGraphicsObject();
        } else {
            if (QGraphicsItem *focusItem = m_view->scene()->focusItem()) {
                if (m_item->isAncestorOf(focusItem)) {
                    focusObject = focusItem->toGraphicsObject();
                }
            }
        }
        //qDebug() << "QAccessibleDeclarativeItem navigate QAccessible::FocusChild" << rel << entry;
        if (focusObject) {
            *target = instantiateObject(focusObject, m_view);
            return 0;
        }
    }
    default: break;
    }

    return -1;

}

int QAccessibleGraphicsViewImplementation::indexOfChild(const QAccessibleInterface *iface) const
{
    // ### No QAccessibleInterfaces are created with a QGraphicsItem.
    // However, we want to support QML, not QGraphicsView in general.
    // And since the UI is written in QML, this means we can assume that *all*
    // QGraphicsItems are actually QGraphicsObjects

    const QGraphicsObject *childObj = static_cast<QGraphicsObject*>(iface->object());
    if (m_item == childObj)
        return 0;

    QList<QGraphicsItem*> kids = m_item->childItems();
    int index = kids.indexOf(const_cast<QGraphicsItem*>(static_cast<const QGraphicsItem*>(childObj)));
    if (index != -1) {
        ++index;
    }
    return index;
}

QFlags<QAccessible::StateFlag> QAccessibleGraphicsViewImplementation::state() const
{
    QAccessible::State state = QAccessible::Normal;

    if (m_item->hasFocus()) {
        state |= QAccessible::Focused;
    }
    return state;
}

bool QAccessibleGraphicsViewImplementation::isAccessible() const
{
    return true;
}

